==> open-xchange-usm_7.0.1.orig.tar.bz2: open-xchange-usm-7.0.1/com.openexchange.usm.api/src/com/openexchange/usm/api/contenttypes/ContentType.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.api.contenttypes;

import java.io.Serializable;

import com.openexchange.usm.api.datatypes.PIMAttachment;
import com.openexchange.usm.api.datatypes.PIMAttachments;
import com.openexchange.usm.api.exceptions.DeserializationFailedException;
import com.openexchange.usm.api.exceptions.USMException;
import com.openexchange.usm.api.session.DataObject;
import com.openexchange.usm.api.session.Session;

/**
 * This interface defines all relevant information for content types, i.e.
 * type of objects that can be synchronized.<br>
 * 
 * Providers of ContentTypes have to make sure that their ContentTypes
 * correctly implement hashCode() and equals() since ContentTypes will be
 * stored in Maps.<br>
 * 
 * By convention, the first field of a ContentType has to contain the ID for objects
 * of that ContentType. In the second field, the parent folder ID has to be stored.
 * Using this convention, the USM sync system can make sure that the required fields
 * are always retrieved regardless of the field filter set by protocols. 
 * 
 * @author afe
 *
 */
public interface ContentType extends Serializable {
	/**
	 * @return unique identifier for this content type, e.g. "task", "contacts"
	 */
	String getID();

	/**
	 * unique code for the content type, e.g. "tasks" = 3;
	 * @return
	 */
	int getCode();

	/**
	 * 
	 * @return Array of fields provided by this content type
	 */
	ContentTypeField[] getFields();

	/**
	 * This method is used to create a DataObject for this ContentType. A ContentType
	 * can either use a generic implementation that is normally available through a
	 * separated bundle, or it can provide a more specific implementation that allows
	 * for better access in Java through explicit getter- and setter-methods for its
	 * fields.
	 * 
	 * @param session Session for which the DataObject should be created
	 * @return DataObject for this ContentType
	 */
	DataObject newDataObject(Session session);

	/**
	 * 
	 * @return ContentTypeTransferHandler that is used by this ContentType for communication with an OX server
	 */
	ContentTypeTransferHandler getTransferHandler();

	/**
	 * This method is used by the USM on slow synchronizations to determine possible matches between
	 * client and server objects that have different (or in the case of the client, possibly no) IDs.
	 * This method should examine the current content of both DataObjects and return a value that indicates
	 * the likeliness of a match. A value < 0 means that the objects do not match, a value >= 0 indicates a
	 * possible match. The higher the returned value is, the more likely is a match between the two DataObjects.
	 * Since only DataObjects of the same ContentType can be compared, the ContentType can use any value range
	 * ( >= 0) and is not required to limit the result to a specific range.<br>
	 * The USM sync mechanism compares all client candidates with all server candidates, joins the two with the
	 * highest rating and eliminates them from the candidate list, and repeats this process until no pair with
	 * a rating >= 0 is found.<br>
	 * Note that normally only DataObjects within the same Folder are compared, only Folder elements are usually
	 * compared also if they have different parent folders.
	 * 
	 * @param client DataObject of this ContentType as reported from a client
	 * @param server DataObject of this ContentType as reported from the server
	 * @return < 0 if no match, >= 0 if a match is likely, the higher, the likelier
	 */
	int getMatchRating(DataObject client, DataObject server);

	/**
	 * This method is called by the persistence subsystem of the sync system to restore the field content of a
	 * previously serialized DataObject. Normally this method can simply copy the data into the field content array,
	 * but ContentTypes may change their fields and old serialized information may be persisted. To enable the
	 * USM to handle "old" serialized data, each ContentType can introduce additional code to perform a mapping
	 * from older versions to the current version.
	 * 
	 * @param data data that was deserialized, may contain extra information before the actual fields
	 * @param i starting index to use for field extraction, all following indexes contain field data
	 * @param fieldContent target field array of a DataObject in which the deserialized data should be stored
	 * @throws DeserializationFailedException if for some reason the desrialization can not be performed
	 */
	void deserializeData(Serializable[] data, int i, Object[] fieldContent) throws DeserializationFailedException;
	
	/**
	 * Creates new attachment to a given DataObject from this content type.  
	 * 
	 * @param object
	 * @param attachment
	 * @throws USMException
	 * @return the timestamp of the new created attachment
	 */
    long createNewAttachment(DataObject object, PIMAttachment attachment) throws USMException;
	
    /**
     * Retrieves all attachments to a given object sorted by the given parameter in  the given order.
     * 
     * @param object
     * @param sort The identifier of a column which determines the sort order of the result. 
     * @param order the wanted order - "asc" for ascending, "desc" for descending 
     * @return object containing all attachments and the last modified timestamp
     * @throws USMException
     */
	PIMAttachments getAllAttachments(DataObject object, String sort, String order) throws USMException;
	
	/**
	 * Retrieves all attachments to a given object.
	 * 
	 * @param object
	 * @return object containing all attachments and the last modified timestamp
	 * @throws USMException
	 */
	PIMAttachments getAllAttachments(DataObject object) throws USMException;
	
	/**
	 * Deletes a given list of attachments from a given object. 
	 * 
	 * @param object the object this attachments are attached to
	 * @param attachmentsToDelete list of attachments to delete
	 * @return the timestamp of this modification
	 * @throws USMException
	 */
	long deleteAttachments(DataObject object, PIMAttachment... attachmentsToDelete) throws USMException;
	
	/**
	 * Retrieves the raw byte data of the attachment.
	 * 
	 * @param object the object this attachment is attached to
	 * @param attachmentId the attachment id
	 * @return the raw byte data of the attachment
	 * @throws USMException
	 */
	byte[] getAttachmentData(DataObject object, int attachmentId) throws USMException;
	
	/**
	 * Returns true if the content type supports attachments;
	 * 
	 * @return
	 */
	boolean supportsPIMAttachments();
	
	/**
	 * Returns the object id or the id of the "parent" object. 
	 * For ex. when the object is Exception from Series Appointment it returns the id of the Series App.  
	 * @param object
	 * @return
	 */
	Object getObjectGroupOwner(DataObject object);
	
	/**
	 * Returns true if the content type can be folder elements content type. 
	 * ContenTypes like folder or groups return false because objects from this content type can not be elements in a folder. 
	 * 
	 * @return
	 */
	boolean canBeFolderElementsContentType();
}

==> open-xchange-usm_7.4.0.orig.tar.bz2: open-xchange-usm-7.4.0/com.openexchange.usm.api/src/com/openexchange/usm/api/contenttypes/ContentType.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.api.contenttypes;

import java.io.Serializable;

import com.openexchange.usm.api.datatypes.PIMAttachment;
import com.openexchange.usm.api.datatypes.PIMAttachments;
import com.openexchange.usm.api.exceptions.DeserializationFailedException;
import com.openexchange.usm.api.exceptions.USMException;
import com.openexchange.usm.api.session.DataObject;
import com.openexchange.usm.api.session.Session;

/**
 * This interface defines all relevant information for content types, i.e.
 * type of objects that can be synchronized.<br>
 * 
 * Providers of ContentTypes have to make sure that their ContentTypes
 * correctly implement hashCode() and equals() since ContentTypes will be
 * stored in Maps.<br>
 * 
 * By convention, the first field of a ContentType has to contain the ID for objects
 * of that ContentType. In the second field, the parent folder ID has to be stored.
 * Using this convention, the USM sync system can make sure that the required fields
 * are always retrieved regardless of the field filter set by protocols. 
 * 
 * @author afe
 *
 */
public interface ContentType extends Serializable {
	/**
	 * @return unique identifier for this content type, e.g. "task", "contacts"
	 */
	String getID();

	/**
	 * unique code for the content type, e.g. "tasks" == 4;
	 * @return
	 */
	int getCode();

	/**
	 * 
	 * @return Array of fields provided by this content type
	 */
	ContentTypeField[] getFields();

	/**
	 * This method is used to create a DataObject for this ContentType. A ContentType
	 * can either use a generic implementation that is normally available through a
	 * separated bundle, or it can provide a more specific implementation that allows
	 * for better access in Java through explicit getter- and setter-methods for its
	 * fields.
	 * 
	 * @param session Session for which the DataObject should be created
	 * @return DataObject for this ContentType
	 */
	DataObject newDataObject(Session session);

	/**
	 * 
	 * @return ContentTypeTransferHandler that is used by this ContentType for communication with an OX server
	 */
	ContentTypeTransferHandler getTransferHandler();

	/**
	 * This method is used by the USM on slow synchronizations to determine possible matches between
	 * client and server objects that have different (or in the case of the client, possibly no) IDs.
	 * This method should examine the current content of both DataObjects and return a value that indicates
	 * the likeliness of a match. A value < 0 means that the objects do not match, a value >= 0 indicates a
	 * possible match. The higher the returned value is, the more likely is a match between the two DataObjects.
	 * Since only DataObjects of the same ContentType can be compared, the ContentType can use any value range
	 * ( >= 0) and is not required to limit the result to a specific range.<br>
	 * The USM sync mechanism compares all client candidates with all server candidates, joins the two with the
	 * highest rating and eliminates them from the candidate list, and repeats this process until no pair with
	 * a rating >= 0 is found.<br>
	 * Note that normally only DataObjects within the same Folder are compared, only Folder elements are usually
	 * compared also if they have different parent folders.
	 * 
	 * @param client DataObject of this ContentType as reported from a client
	 * @param server DataObject of this ContentType as reported from the server
	 * @return < 0 if no match, >= 0 if a match is likely, the higher, the likelier
	 */
	int getMatchRating(DataObject client, DataObject server);

	/**
	 * This method is called by the persistence subsystem of the sync system to restore the field content of a
	 * previously serialized DataObject. Normally this method can simply copy the data into the field content array,
	 * but ContentTypes may change their fields and old serialized information may be persisted. To enable the
	 * USM to handle "old" serialized data, each ContentType can introduce additional code to perform a mapping
	 * from older versions to the current version.
	 * 
	 * @param data data that was deserialized, may contain extra information before the actual fields
	 * @param i starting index to use for field extraction, all following indexes contain field data
	 * @param fieldContent target field array of a DataObject in which the deserialized data should be stored
	 * @throws DeserializationFailedException if for some reason the desrialization can not be performed
	 */
	void deserializeData(Serializable[] data, int i, Object[] fieldContent) throws DeserializationFailedException;
	
	/**
	 * Returns true if the content type supports attachments;
	 * 
	 * @return
	 */
	boolean supportsPIMAttachments();
	
	/**
	 * Returns the object id or the id of the "parent" object. 
	 * For ex. when the object is Exception from Series Appointment it returns the id of the Series App.  
	 * @param object
	 * @return
	 */
	Object getObjectGroupOwner(DataObject object);
	
	/**
	 * Returns true if the content type can be folder elements content type. 
	 * ContenTypes like folder or groups return false because objects from this content type can not be elements in a folder. 
	 * 
	 * @return
	 */
	boolean canBeFolderElementsContentType();
}
